#!/usr/bin/env tclsh
# -*- tcl -*-
# Run an entire tournament of bots in a game world.
# Arguments
# - Directory holding the game files (including RunGame.tcl)
# - Paths to two bots.
# - Number of games/iterations to run.

# This is like/similar to the core RunGame. Except that the specified
# bots play multiple times against each other to better qualify their
# relative strength. Another difference is that the bot names are
# derived from their file names. With qualifiers if a bot is played
# against itself.

package require Tcl 8.6
package require fileutil

proc main {} {
    cmdline
    gameon
}

proc cmdline {} {
    global argv world player1 player2 iter
    if {[llength $argv] ni {3 4}} usage
    lassign $argv world player1 player2 iter

    set world   [file normalize $world]
    set player1 [file normalize $player1]
    set player2 [file normalize $player2]

    if {($iter eq {}) ||
	![string is integer $iter] ||
	($iter < 1)} {
	set iter 50
    }
    return
}

proc usage {} {
    global argv0
    puts stderr "Usage: $argv0 worlddir player1 player2 ?num-games?"
    exit 1
}

proc stop {args} {
    global argv0
    puts stderr "$argv0 error: $args"
    exit 1
}

proc gameon {} {
    global world player1 player2 iter na nb

    set na [file rootname [file tail $player1]]
    set nb [file rootname [file tail $player2]]
    if {$na eq $nb} {
	append na -A
	append nb -B
    }

    lappend players $na $nb
    initranks $players

    cd $world
    file delete -force logs
    file mkdir         logs

    set counter 1
    while {$iter} {
	play $counter
	incr counter
	incr iter -1
    }
    # Reset iter for display, counter now contains the value.
    set  iter $counter
    incr iter -1

    showranks
    return
}

proc initranks {players} {
    global points
    foreach p $players {
	dict set points $p 0
    }
    return
}

proc showranks {} {
    global points iter

    # invert
    dict for {player have} $points {
	dict lappend rank $have $player
    }

    # normalize points to iter, then to percentages*100 (i.e. with 2
    # fractional digits in the fixed point number).  note that this
    # may merge rankings previously separate.
    array set ranking {}
    dict for {have plist} $rank {
	set score [expr {int(10000*double($have)/$iter)}]
	lappend ranking($score) {*}$plist
    }

    # and finally show things. larger number is better.
    parray ranking
    return
}

proc record {pa pb winner} {
    global points
    if {$winner eq "tie"} {
	dict incr points $pa 1
	dict incr points $pb 1
    } elseif {$winner eq $pa} {
	dict incr points $pa 2
    } elseif {$winner eq $pb} {
	dict incr points $pb 2
    } else {
	stop Bad winner <$winner>
    }
    return
}

proc play {game} {
    global player1 player2 na nb

    puts -nonewline "\[[format %3d $game]\] Playing: \"$na\" vs \"$nb\" ... "
    flush stdout

    set label game$game
    set log logs/${label}.log

    # Note: We ignore the low-level log.
    # This code assumes the modified RunGame.tcl with the higher-level log in stderr.
    exec 2> $log > $log.proto \
	[info nameofexecutable] RunGame.tcl $label . WorldMap.tcl \
	$na $player1 \
	$nb $player2

    # The winner of the game (ortie) is in the 2nd to last line.
    set result [string trim [lindex [split [fileutil::cat $log] \n] end-2]]
    puts $result

    if {$result eq "tie"} {
	record $na $nb tie
    } else {
	# Line = ('winner' $turns $name)
	record $na $nb [lindex $result 2]
    }
}

main
